/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: Block.h,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */
#ifndef OpenSOAP_Block_H
#define OpenSOAP_Block_H

#include <OpenSOAP/XMLElm.h>
#include <OpenSOAP/XMLNamespace.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

    typedef struct tagOpenSOAPBlock OpenSOAPBlock;
    typedef OpenSOAPBlock *OpenSOAPBlockPtr;

    int
    OPENSOAP_API
    OpenSOAPBlockGetValueMB(OpenSOAPBlockPtr /* [in] */ soap_block,
                            const char * /* [in] */ type_name,
                            void * /* [out] */ value);

    int
    OPENSOAP_API
    OpenSOAPBlockGetValueWC(OpenSOAPBlockPtr /* [in] */ soap_block,
                            const wchar_t * /* [in] */ type_name,
                            void * /* [out] */ value);

    int
    OPENSOAP_API
    OpenSOAPBlockSetValueMB(OpenSOAPBlockPtr /* [in] */ soap_block,
                            const char * /* [in] */ type_name,
                            void * /* [in] */ value);

    int
    OPENSOAP_API
    OpenSOAPBlockSetValueWC(OpenSOAPBlockPtr /* [in] */ soap_block,
                            const wchar_t * /* [in] */ type_name,
                            void * /* [in] */ value);

    int
    OPENSOAP_API
    OpenSOAPBlockSetNamespaceMB(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                                const char * /* [in] */ ns_uri,
                                const char * /* [in] */ ns_prefix);

    int
    OPENSOAP_API
    OpenSOAPBlockSetNamespaceWC(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                                const wchar_t * /* [in] */ ns_uri,
                                const wchar_t * /* [in] */ ns_prefix);

    int
    OPENSOAP_API
    OpenSOAPBlockGetNamespace(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                              OpenSOAPXMLNamespacePtr * /* [out] */ ns);

    int
    OPENSOAP_API
    OpenSOAPBlockIsSameNamespaceMB(OpenSOAPBlockPtr /* [in] */ soap_block,
                                   const char * /* [in] */ ns_uri,
                                   int * /* [out] */ is_same_uri);

    int
    OPENSOAP_API
    OpenSOAPBlockIsSameNamespaceWC(OpenSOAPBlockPtr /* [in] */ soap_block,
                                   const wchar_t * /* [in] */ ns_uri,
                                   int * /* [out] */ is_same_uri);

    int
    OPENSOAP_API
    OpenSOAPBlockAddAttributeMB(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                                const char * /* [in] */ attr_name,
                                const char * /* [in] */ attr_type,
                                void * /* [in] */ attr_value,
                                OpenSOAPXMLAttrPtr * /* [out] */ attr);

    int
    OPENSOAP_API
    OpenSOAPBlockAddAttributeWC(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                                const wchar_t * /* [in] */ attr_name,
                                const wchar_t * /* [in] */ attr_type,
                                void * /* [in] */ attr_value,
                                OpenSOAPXMLAttrPtr * /* [out] */ attr);

    int
    OPENSOAP_API
    OpenSOAPBlockGetAttributeMB(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                                const char * /* [in] */ attr_name,
                                OpenSOAPXMLAttrPtr * /* [out] */ attr);
    
    int
    OPENSOAP_API
    OpenSOAPBlockGetAttributeWC(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                                const wchar_t * /* [in] */ attr_name,
                                OpenSOAPXMLAttrPtr * /* [out] */ attr);
    
    int
    OPENSOAP_API
    OpenSOAPBlockGetChildValueMB(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                                 const char * /* [in] */ p_name,
                                 const char * /* [in] */ type_name,
                                 void * /* [out] */ value);

    int
    OPENSOAP_API
    OpenSOAPBlockGetChildValueWC(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                                 const wchar_t * /* [in] */ p_name,
                                 const wchar_t * /* [in] */ type_name,
                                 void * /* [out] */ value);

    int
    OPENSOAP_API
    OpenSOAPBlockSetChildValueMB(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                                 const char * /* [in] */ p_name,
                                 const char * /* [in] */ type_name,
                                 void * /* [in] */ value);

    int
    OPENSOAP_API
    OpenSOAPBlockSetChildValueWC(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                                 const wchar_t * /* [in] */ p_name,
                                 const wchar_t * /* [in] */ type_name,
                                 void * /* [in] */ value);

    int
    OPENSOAP_API
    OpenSOAPBlockGetNextChild(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                              OpenSOAPXMLElmPtr * /* [in, out] */ xml_elm);

    int
    OPENSOAP_API
    OpenSOAPBlockAddChildMB(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                            const char * /* [in] */ elm_name,
                            OpenSOAPXMLElmPtr * /* [out] */ xml_elm);

    int
    OPENSOAP_API
    OpenSOAPBlockAddChildWC(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                            const wchar_t * /* [in] */ elm_name,
                            OpenSOAPXMLElmPtr * /* [out] */ xml_elm);

    int
    OPENSOAP_API
    OpenSOAPBlockGetChildMB(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                            const char * /* [in] */ elm_name,
                            OpenSOAPXMLElmPtr * /* [out] */ xml_elm);

    int
    OPENSOAP_API
    OpenSOAPBlockGetChildWC(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                            const wchar_t * /* [in] */ elm_name,
                            OpenSOAPXMLElmPtr * /* [out] */ xml_elm);

    int
    OPENSOAP_API
    OpenSOAPBlockGetMustunderstandAttr(OpenSOAPBlockPtr /* [in] */ soap_block,
                                       int * /* [out] */ must_std);

    int
    OPENSOAP_API
    OpenSOAPBlockSetMustunderstandAttr(OpenSOAPBlockPtr /* [in, out] */ soap_block);

    int
    OPENSOAP_API
    OpenSOAPBlockClearMustunderstandAttr(OpenSOAPBlockPtr /* [in, out] */ soap_block);

    int
    OPENSOAP_API
    OpenSOAPBlockGetActorAttr(OpenSOAPBlockPtr /* [in] */ soap_block,
                              OpenSOAPStringPtr * /* [out] */ actor_url);

    int
    OPENSOAP_API
    OpenSOAPBlockSetActorAttrMB(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                                const char * /* [in] */ actor_url);

    int
    OPENSOAP_API
    OpenSOAPBlockSetActorAttrWC(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                                const wchar_t * /* [in] */ actor_url);

    int
    OPENSOAP_API
    OpenSOAPBlockClearActorAttr(OpenSOAPBlockPtr /* [in, out] */ soap_block);

    int
    OPENSOAP_API
    OpenSOAPBlockSetActorAttrNext(OpenSOAPBlockPtr /* [in, out] */ soap_block);
    
    int
    OPENSOAP_API
    OpenSOAPBlockIsActorAttrNext(OpenSOAPBlockPtr /* [in] */ soap_block,
                                 int * /* [out] */ is_actor_next);

    int
    OPENSOAP_API
    OpenSOAPBlockGetName(OpenSOAPBlockPtr /* [in] */ block,
                         OpenSOAPStringPtr * /* [out] */ name);
    
    int
    OPENSOAP_API
    OpenSOAPBlockGetCharEncodingString(OpenSOAPBlockPtr /* [in] */ soap_block,
                                        const char * /* [in] */ ch_enc,
                                        OpenSOAPByteArrayPtr /* [out] */ b_ary);
#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* OpenSOAP_Block_H */
