/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: MsgDrvCreator_main.cpp,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */
#if defined(WIN32)
#include <windows.h>
#include <io.h>
#else
#include <unistd.h>
#endif

#include <fstream>
#include <string>

#include "ServerCommon.h"
#include "MsgDrvCreator.h"
#if !defined(NO_LOGGER)
#include "Logger.h"
#endif

using namespace OpenSOAP;
using namespace std;

int
main(int argc, char** argv) {

#if defined(NO_LOGGER)
	string logFile = OPENSOAP_TMP_PATH + "log/msgDrvCreator";
	if (argc >1) {
		logFile = argv[1];
	}
	FILE* outfp = fopen(logFile.c_str(), "w");
	if (!outfp) {
		return 1;
	}
	if (argc <=1) {
		fprintf(outfp, "invalid initial argument. argc=(%d)\n", argc);
		fprintf(outfp, "exit.\n");
		fclose(outfp);
		return EXIT_FAILURE;
	}
#if 1
	int old;
	int olderr;
	//backup stdout
	old = _dup(1);
	//stdout -> outfp
	if (-1 == _dup2(_fileno(outfp), 1)) {
		fprintf(outfp, "_dup2 to stdout failed.\n");
		fprintf(outfp, "exit.\n");
		fclose(outfp);
		return EXIT_FAILURE;
	}
	//backup stderr
	olderr = _dup(2);
	//stderr -> outfp
	if (-1 == _dup2(_fileno(outfp), 2)) {
		fprintf(outfp, "_dup2 to stderr failed.\n");
		fprintf(outfp, "exit.\n");
		fclose(outfp);
		return EXIT_FAILURE;
	}
#endif
#else
  // set log level
  Logger::InitOpt opt;
  opt.outputLevel = LOG_DEBUG;
  Logger::init(opt);
  //write start message
  Logger::printLog(NULL, " MsgDrvCreator_main start", LOG_INFO,
		   0x00100003L, "exe");
#endif

#if !defined(WIN32)
  // make it to daemon & write pid file
#if HAVE_DAEMON
  daemon(0, 1);
#endif

  // write pid file
  pid_t pid = getpid();
  ofstream pidFile(MSGDRV_PID_FILE.c_str());
  if (!pidFile) {
    cerr << "Error: pid file [" 
	 << MSGDRV_PID_FILE << "] cannot open!" << endl;
    exit(1);
  }
  pidFile << pid;
  pidFile.close();
#endif

  // create instance of Message Driver and run  
  ChannelManager* chnlMngr = new MsgDrvCreator();
  chnlMngr->setMask(0x000);
  chnlMngr->run();
  
  delete chnlMngr;
  
#if defined(NO_LOGGER)
#if 1
	fclose(outfp);
	_dup2(old, 1);
	_dup2(olderr, 2);
#endif
#else
  // write stop message
  Logger::printLog(NULL, " MsgDrvCreator_main stop", LOG_INFO,
		   0x00100003L, "exe");
  Logger::term();
#endif

  return EXIT_SUCCESS;
}
