/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: SrvConfAttrMgr.cpp,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#if HAVE_CONFIG_H
# include "config.h"
#endif

#if defined(WIN32)
#include <process.h>
#else
#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>

#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/sem.h>

#include <sys/socket.h>
#include <sys/un.h>
#include <unistd.h>
#endif

#include <stdexcept>
#include <string>
#include <iostream>

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif /* HAVE_CONFIG_H */

/* libxml2 */
#if defined(HAVE_LIBXML2_XMLVERSION_H) 
#include <libxml2/tree.h>
#include <libxml2/parser.h>
#else
#include <libxml/tree.h>
#include <libxml/parser.h>
#endif /* !HAVE_LIBXML2_XMLVERSION_H */

#include "MapUtil.h"
#include "ServerCommon.h"
//#include "SrvConfAttrMgrProtocol.h"
#include "SSMLAttrMgrProtocol.h"
#include "XmlQuery.h"
#include "SrvConfAttrMgr.h"

using namespace OpenSOAP;
using namespace std;

const std::string SrvConfAttrMgr::CONF_FILE_SUFFIX = "conf";

// for LOG
static const std::string CLASS_SIG = "SrvConfAttrMgr";

//UNIXhC\Pbg̃AhXXMLt@CfBNgw肷
SrvConfAttrMgr::SrvConfAttrMgr(
#if defined(WIN32)
								const int
#else
								const std::string& 
#endif
									socketAddr, 
								const std::string& xmlDir)
  : socketAddr_(socketAddr)
    , xmlDir_(xmlDir)
{
  //mutex̏
#if defined(WIN32)
	loadXmlLock_ = CreateMutex(NULL, FALSE, "SSMLATTRMGRMTX");
#else
  int status = pthread_mutex_init(&loadXmlLock_, NULL);
  if (0 != status) {
    //throw OpenSOAPException();
    throw std::exception();
  }
#endif

  //fBNgpX/ŏI悤ɂĂ
  if (FILE_PATH_DELMIT != xmlDir_.at(xmlDir_.length() - 1)) {
    xmlDir_ += FILE_PATH_DELMIT;
  }
#ifdef DEBUG
  cerr << CLASS_SIG << "::" << CLASS_SIG << ": xmlDir_=["
       << xmlDir_ << "]" << endl;
#endif /* DEBUG */

  //SEM init
#if defined(WIN32)
	semid_ = CreateSemaphore(NULL, 1, 1, "SRVCONFATTRMGRSEM");
	//shm
	if (-1 == createShm()) {
	    throw std::exception();
	}
#else
  key_t semKey = ftok(SRVCONFATTRMGR_SOCKET_ADDR.c_str(), 1);
  semid_ = semget(semKey, 1, IPC_CREAT|0666);
  if (-1 == semid_) {
    throw std::exception();
  }

#if defined(__GNU_LIBRARY__) && !defined(_SEM_SEMUN_UNDEFINED)
  union semun semUn;
#else
  union semun {
    int val;
    struct semid_ds *buf;
    unsigned short int *array;
    struct seminfo *__buf;
  } semUn;
#endif

  if (-1 == semctl(semid_, 0, GETVAL, semUn)) {
    throw std::exception();
  }
  semUn.val = 0;
  if (-1 == semctl(semid_, 0, SETVAL, semUn)) {
    throw std::exception();
  }
#endif

  //SSMLt@C̃[h
  if (0 != loadXml()) {
    cerr << TAG_ERR << CLASS_SIG << "::" << CLASS_SIG 
	 << ": load ssml file failed" << endl;
    throw std::exception();
  }
}

void 
SrvConfAttrMgr::freeXmlList()
{
  //xmlList_DOM\J
  std::list<xmlDocPtr>::iterator pos;
  for (pos = xmlList_.begin(); pos != xmlList_.end(); ++pos) {
    xmlFreeDoc(*pos);
  }
  xmlList_.clear();

}

SrvConfAttrMgr::~SrvConfAttrMgr()
{
  //xmlList_DOM\J
  freeXmlList();

  // mutex j
#if defined(WIN32)
	CloseHandle(loadXmlLock_);
#else
  int status = pthread_mutex_destroy(&loadXmlLock_);
  if (0 != status) {
    //throw OpenSOAPException();
    //throw std::exception();
    cerr << CLASS_SIG << "::" << CLASS_SIG
	 << ": mutext destroy failed" << endl;
  }
#endif

#if defined(WIN32)
	deleteShm();
	CloseHandle(semid_);
#endif
}

//ʐMJn
int SrvConfAttrMgr::run()
{
  //===================================================================
  // ʐM SrvConfAttrMgr <=> SSMLAttrHandler
  //===================================================================
#if defined(WIN32)
	typedef SOCKADDR SockAddr;
	typedef SOCKADDR_IN SockAddrAf;
	//SOCKADDR_IN serverAddr;
	SockAddrAf serverAddr;
	SockAddrAf clientAddr;

	SOCKET sockfd = INVALID_SOCKET;
	SOCKET newsockfd = INVALID_SOCKET;
	WSADATA data;
	if (WSAStartup(0x101,&data)) {
		cerr << "** E **: WSAStartup failed." << endl;
		throw exception();
	}
	int af = AF_INET;
	int addrLen = sizeof(SockAddrAf);
	int clientLen = int();
#else
  typedef struct sockaddr SockAddr;
  typedef struct sockaddr_un SockAddrAf;

  int af = AF_UNIX;
  int sockfd = int();
  int newsockfd = int();
  SockAddrAf serverAddr;
  SockAddrAf clientAddr;
#if HAVE_TYPE_OF_ADDRLEN_AS_SOCKLEN_T
  socklen_t addrLen = socklen_t(); //socket_t();
  socklen_t clientLen = socklen_t();
#endif
#if HAVE_TYPE_OF_ADDRLEN_AS_SIZE_T
  size_t addrLen = size_t();
  size_t clientLen = size_t();
#endif
#if HAVE_TYPE_OF_ADDRLEN_AS_INT
  int addrLen = int();
  int clientLen = int();
#endif

#endif

  if (0 > (sockfd = socket(af, SOCK_STREAM, 0))) {
    cerr << TAG_ERR << CLASS_SIG << "::run: open socket failed" << endl;
    return -1;
  }

#ifdef DEBUG
  cerr << CLASS_SIG << "::run" 
       << ": socket open OK!" << endl;
#endif

  memset((char*)&serverAddr, 0, sizeof(serverAddr));

#if defined(WIN32)
	serverAddr.sin_family = af;
	serverAddr.sin_addr.s_addr = INADDR_ANY;
	serverAddr.sin_port = htons(socketAddr_);
#else
  serverAddr.sun_family = af;
  strcpy(serverAddr.sun_path, socketAddr_.c_str());
  addrLen = sizeof(serverAddr.sun_family) + strlen(serverAddr.sun_path);

  unlink(socketAddr_.c_str());
#endif

  if (0 > bind(sockfd, (SockAddr*)&serverAddr, addrLen)) {
    cerr << TAG_ERR << CLASS_SIG << "::run"
	 << ": bind failed" << endl;
    return -1;
  }

#ifdef DEBUG
  cerr << CLASS_SIG << "::run" 
       << ": bind OK!" << endl;
#endif

  if (0 > listen(sockfd, 5)) {
    cerr << TAG_ERR << CLASS_SIG << "::run"
	 << ": listen failed" << endl;
    return -1;
  }

#ifdef DEBUG
  cerr << CLASS_SIG << "::run" 
       << ": listen OK!" << endl;
#endif
    
  for (;;) {
    clientLen = sizeof(clientAddr);

#ifdef DEBUG
    cerr << CLASS_SIG << "::run" 
	 << ": accept ready!" << endl;
#endif
    
    newsockfd = accept(sockfd, (SockAddr*)&clientAddr, &clientLen);
    if (0 > newsockfd) {
      cerr << TAG_ERR << CLASS_SIG << "::run"
	   << ": accept failed" << endl;
      return -1;
    }

#ifdef DEBUG
    cerr << CLASS_SIG << "::run" 
	 << ": accept OK! fd=(" << newsockfd << ")" << endl;
#endif

    //XbhɓnKv̂f[^
    ThrData* thrData = new ThrData();
    thrData->that = this;
    thrData->sockfd = newsockfd;

    //ʐMXbh쐬
#if defined(WIN32)
	long status = _beginthread(connectionThread, 0, thrData);
#else
    pthread_t thrId;
    int status = pthread_create(&thrId,
				NULL,
				connectionThread,
				(void*)thrData);
    if (0 != status) {
      cerr << TAG_ERR << CLASS_SIG << "::run"
	   << ": create thread failed" << endl;
      return -1;
    }
#endif
  } // end for (;;)
}


//XMLt@C̓ǂݍ
int SrvConfAttrMgr::loadXml()
{ 
#ifdef DEBUG
  cerr << CLASS_SIG << "::loadXml()" << endl;
#endif /* DEBUG */

  // mutex lock
  // bNݒ
#if defined(WIN32)
	WaitForSingleObject(loadXmlLock_, INFINITE);
#else  
  int status = pthread_mutex_lock(&loadXmlLock_);
  if (0 != status) {
    //error
    cerr << TAG_ERR << CLASS_SIG << "::loadXml: mutex lock failed" << endl;
    return -1;
  }
#endif

  //xmlList_DOM\J
  freeXmlList();

  typedef struct direct Dir;

#if defined(WIN32)
	HANDLE h;
	WIN32_FIND_DATA fData;
	string fPattern(xmlDir_);
	fPattern += "*";			// find pattern = "dir/*"
	h = FindFirstFile(fPattern.c_str(), &fData);

#ifdef DEBUG
cerr << "Find path=[" << fPattern << "]" << endl;
cerr << "FindFirstFile: [" << fData.cFileName << "]" << endl;
#endif

#else
  DIR* dp = NULL;
  dirent* dir = NULL;
  //wfBNgJ
  if (NULL == (dp = opendir(xmlDir_.c_str()))) {
    cerr << TAG_ERR << CLASS_SIG << "::loadXml: cannot open directory.[" 
	 << xmlDir_ << "]" << endl;
    // "open dir failed " + fifoDirPath

    // bN
    status = pthread_mutex_unlock(&loadXmlLock_);
    if (0 != status) {
      cerr << TAG_ERR << CLASS_SIG << "::loadXml: mutex unlock failed" << endl;
      return -1;
    }
    return -1;
  }
#endif

  std::string srvConfFilePath;

#if defined(WIN32)
	//directory check
	if (!(fData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)) {
	    //full path file name
	    srvConfFilePath = xmlDir_ + fData.cFileName;
		addXmlList(srvConfFilePath);
	}

	while (FindNextFile(h, &fData)) {
#ifdef DEBUG
cerr << "** FindNextFile: [" << fData.cFileName << "] ***" << endl;
#endif
		if (fData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
			cerr << "is dir. [" << fData.cFileName << "]" << endl;
			continue;
		}
	    //full path file name
	    srvConfFilePath = xmlDir_ + fData.cFileName;
		addXmlList(srvConfFilePath);
	}
	FindClose(h);

#else
  while (NULL != (dir = readdir(dp))) {
    if (0 == dir->d_ino) {
      cerr << "0 == dir->d_ino" << endl;
      continue;
    }

    //full path file name
    srvConfFilePath = xmlDir_ + dir->d_name;
	addXmlList(srvConfFilePath);
  }
  closedir(dp);
#endif

  // shm buffering clear
  clearShm();

  // bN
#if defined(WIN32)
	ReleaseMutex(loadXmlLock_);
#else
  status = pthread_mutex_unlock(&loadXmlLock_);
  if (0 != status) {
    cerr << TAG_ERR << CLASS_SIG << "::loadXml: mutex unlock failed" << endl;
    return -1;
  }
#endif

  return 0;
}

bool
SrvConfAttrMgr::addXmlList(const string& srvConfFilePath) 
{
    //t@Cgq̃`FbN
    if (CONF_FILE_SUFFIX != 
		srvConfFilePath.substr(srvConfFilePath.rfind(".")+1)) {
		return false;
    }

#ifdef DEBUG
    cerr << CLASS_SIG << "::addXmlList: xmlParseFile : file=["
	 << srvConfFilePath << "]" << endl;
#endif /* DEBUG */

    /* parse ssml begin */
    /* COMPAT: Do not generate nodes for formatting spaces */
    LIBXML_TEST_VERSION
      xmlKeepBlanksDefault(0);
    xmlDocPtr doc = NULL;
    xmlNodePtr node1 = NULL;
  
    /* build an XML tree from a file */

    doc = xmlParseFile((const char*)(srvConfFilePath.c_str()));
    if (doc == NULL) {
      cerr << TAG_WRN << CLASS_SIG << "::addXmlList: "
	   << "parsing file " << srvConfFilePath << " failed." << endl;
		return false;
    }
    
    /* check document (root node) */
    node1 = xmlDocGetRootElement(doc);
    if (node1 == NULL) {
      cerr << TAG_WRN << CLASS_SIG << "::addXmlList: "
	   << "empty document " << srvConfFilePath << endl;
      xmlFreeDoc(doc);
		return false;
    }
    //Xg֒ǉ
    xmlList_.push_back(doc);

#ifdef DEBUG
    cerr << CLASS_SIG << "::addXmlList: add doc." << endl;
#endif /* DEBUG */

	return true;
}


//ʐMXbh
#if defined(WIN32)
void
#else
void* 
#endif
SrvConfAttrMgr::connectionThread(void* arg)
{
#ifdef DEBUG
  cerr << CLASS_SIG << "::connectionThread()" << endl;
#endif /* DEBUG */

  //ñ`FbN
  if (!arg) {
    cerr << TAG_ERR << CLASS_SIG 
	 << "::connectionThread: invalid argument" << endl;
#if defined(WIN32)
	_endthread();
#else
    pthread_exit(NULL);
#endif
  }

  //n̓WJ
  ThrData* thrData = (ThrData*)arg;
  SrvConfAttrMgr* that = thrData->that;
  int sockfd = thrData->sockfd;
  delete thrData;

#ifdef DEBUG
  cerr << CLASS_SIG << "::connectionThread: sockfd=(" << sockfd << ")" << endl;
#endif /* DEBUG */

  that->connectProc(sockfd);

  if (0 < sockfd) {
#if defined(WIN32)
	closesocket(sockfd);
#else
    close(sockfd);
#endif
  }

#if defined(WIN32)
	_endthread();
#else
  pthread_exit(NULL);
  return NULL;
#endif
}

//ʐM
void SrvConfAttrMgr::connectProc(int sockfd)
{
#ifdef DEBUG
  cerr << CLASS_SIG << "::connectProc() arg=[" << sockfd << "]" << endl;
#endif /* DEBUG */

  //f[^M
  std::string readBuf;
  if (0 > read(sockfd, readBuf)) {
    cerr << TAG_ERR << CLASS_SIG 
	 << "::connectProc: read failed" << endl;
    return;
  }

#ifdef DEBUG
  cerr << CLASS_SIG << "::connectProc: read=["
       << readBuf << "]" << endl;
#endif

  //Mf[^̉

  MapUtil readDataMap(ITEM_DELMIT, VAL_DELMIT);
  readDataMap.insertItem(readBuf);

#ifdef DEBUG
  readDataMap.spy();
#endif /* DEBUG */

  //ԐMf[^
  std::string response;
  std::string retCode;
  MapUtil writeDataMap(ITEM_DELMIT, VAL_DELMIT);

  
  //R}h`FbN
  if (!readDataMap.exists(CMD)) {
    cerr << TAG_ERR << CLASS_SIG 
	 << "::connectProc: read failed" << endl;
    writeDataMap.insert(make_pair(RESULT, FAILURE));
    retCode = CMD + " not found";
    writeDataMap.insert(make_pair(RET_CODE, retCode));
    write(sockfd, writeDataMap.toString());

    return;
  }
  //R}heo
  std::string cmd = readDataMap[CMD];

  if (RELOAD_XML == cmd) {
    //SSMLt@C̃[h
    if (0 != loadXml()) {
      cerr << TAG_ERR << CLASS_SIG 
	   << "::connectProc: reload SSML failed" << endl;
      writeDataMap.insert(make_pair(RESULT, FAILURE));
      retCode = "reload failed";
      writeDataMap.insert(make_pair(RET_CODE, retCode));
      write(sockfd, writeDataMap.toString());

      return;
    }
    writeDataMap.insert(make_pair(RESULT, SUCCESS));
    write(sockfd, writeDataMap.toString());

    return;
  }
  else if (SEARCH == cmd) {
    //T[`f[^`FbN
    if (!readDataMap.exists(QUERY)) {
      cerr << TAG_ERR << CLASS_SIG 
	   << "::connectProc: search query not found." << endl;

      writeDataMap.insert(make_pair(RESULT, FAILURE));
      retCode = QUERY + " not found";
      writeDataMap.insert(make_pair(RET_CODE, retCode));
      write(sockfd, writeDataMap.toString());

      return;
    }
    //T[`eo
    std::string query = readDataMap[QUERY];

#ifdef DEBUG
    cerr << CLASS_SIG << "::connectProc: search query=[" << query << "]" << endl;
#endif /* DEBUG */

    response = queryXml(query);

#ifdef DEBUG
    cerr << CLASS_SIG << "::connectProc: queryXml response=["
	 << response << "]" << endl;
#endif /* DEBUG */
    
    write(sockfd, response);

    return;
    
  }
  else {
    cerr << TAG_ERR << CLASS_SIG 
	 << "::connectProc: invalid command. cmd=[" << cmd << "]" << endl;

    writeDataMap.insert(make_pair(RESULT, FAILURE));
    retCode = CMD + " invalid";
    writeDataMap.insert(make_pair(RET_CODE, retCode));
    write(sockfd, writeDataMap.toString());

    return;
  }
}

std::string SrvConfAttrMgr::queryXml(const std::string& query) 
{
#ifdef DEBUG
  cerr << CLASS_SIG << "::queryXml() arg=[" << query << "]" << endl;
#endif /* DEBUG */

  XmlQuery* xmlQuery = new XmlQuery(query);
  
  //mutex lock check.

#ifdef DEBUG
  cerr << "XmlQuery create done. doc size=(" 
       << xmlList_.size() << ")" << endl;
#endif /* DEBUG */

  std::vector<std::string> values;
  std::list<xmlDocPtr>::const_iterator pos;
  for (pos = xmlList_.begin(); pos != xmlList_.end(); ++pos) {
    xmlDocPtr doc = *pos;
    xmlNodePtr node = xmlDocGetRootElement(doc);
#ifdef DEBUG
    cerr << "### call getValue ###" << endl;
#endif /* DEBUG */

    int ret = xmlQuery->getValue(doc, node, values);

#ifdef DEBUG
    cerr << "xmlQuery ret=(" << ret << ")" << endl;
#endif /* DEBUG */

    if (0 == ret && !xmlQuery->isMulti()) {
      //if (0 == xmlQuery->getValue(doc, node, values)) {
      break;
    }
  }

  MapUtil writeDataMap(ITEM_DELMIT, VAL_DELMIT);

  if (0 < values.size()) {
    writeDataMap.insert(make_pair(RESULT, SUCCESS));
    std::string answer;
    std::vector<std::string>::const_iterator vpos;
    for (vpos = values.begin(); vpos != values.end(); ++vpos) {
      if (!answer.empty()) {
	answer += SUB_ITEM_DELMIT;
      }
      answer += *vpos;
    }
    writeDataMap.insert(make_pair(ANSWER, answer));

    //shm buffering
    addShm(query, answer);
  }
  else {
    writeDataMap.insert(make_pair(RESULT, FAILURE));
    std::string retCode("querey failed");
    writeDataMap.insert(make_pair(RET_CODE, retCode));
  }

  delete xmlQuery;

  return writeDataMap.toString();
}

void SrvConfAttrMgr::clearShm()
{
  // SEM lock
#if defined(WIN32)
	WaitForSingleObject(semid_, INFINITE);
#else
  struct sembuf semBuf;
  // semVal -> 1
  semBuf.sem_num = 0;
  semBuf.sem_op = 1;
  semBuf.sem_flg = 0;
  if (-1 == semop(semid_, &semBuf, 1)) {
    cerr << TAG_ERR << CLASS_SIG << "::clearShm: semop up failed" << endl;
    return;
  }
#endif

  char* shm = NULL;

#if defined(WIN32)
	shm = (char*)MapViewOfFile(shmMap, FILE_MAP_ALL_ACCESS, 0, 0, 0);
#else
  key_t shm_key = ftok(SRVCONFATTRMGR_SOCKET_ADDR.c_str(), 0);
  const int SHMSZ = 196608;
  int shmid = 0;

  if ((shmid = shmget(shm_key, SHMSZ, IPC_CREAT|0666)) < 0) {
    cerr << TAG_ERR << CLASS_SIG << "::addShm: shmget failed." << endl;
    return;
  }

  if ((shm = (char*)shmat(shmid, NULL, 0)) == (char*)-1) {
    cerr << TAG_ERR << CLASS_SIG << "::addShm: shmat failed." << endl;
    return ;
  }
#endif

  *shm = 0x00;

#if defined(WIN32)
	UnmapViewOfFile(shm);
#else
  shmdt(shm);
#endif

  // SEM unlock
#if defined(WIN32)
	ReleaseSemaphore(semid_, 1, NULL);
#else
  // semVal -> 0
  semBuf.sem_num = 0;
  semBuf.sem_op = -1;
  semBuf.sem_flg = 0;
  if (-1 == semop(semid_, &semBuf, 1)) {
    cerr << TAG_ERR << CLASS_SIG << "::clearShm: semop down failed" << endl;
    return;
  }
#endif
}

void SrvConfAttrMgr::addShm(const std::string& query, const std::string& value)
{
  // SEM lock
#if defined(WIN32)
	WaitForSingleObject(semid_, INFINITE);
#else
  struct sembuf semBuf;
  // semVal -> 1
  semBuf.sem_num = 0;
  semBuf.sem_op = 1;
  semBuf.sem_flg = 0;
  if (-1 == semop(semid_, &semBuf, 1)) {
    cerr << TAG_ERR << CLASS_SIG << "::addShm: semop up failed" << endl;
    return;
  }
#endif

  char* shm = NULL;
  const int SHMSZ = 196608;
  char shmBuf[SHMSZ];

#if defined(WIN32)
	shm = (char*)MapViewOfFile(shmMap, FILE_MAP_ALL_ACCESS, 0, 0, 0);
#else
  key_t shm_key = ftok(SRVCONFATTRMGR_SOCKET_ADDR.c_str(), 0);
  int shmid = 0;

  if ((shmid = shmget(shm_key, SHMSZ, IPC_CREAT|0666)) < 0) {
    cerr << TAG_ERR << CLASS_SIG << "::addShm: shmget failed." << endl;
    return;
  }

  if ((shm = (char*)shmat(shmid, NULL, 0)) == (char*)-1) {
    cerr << TAG_ERR << CLASS_SIG << "::addShm: shmat failed." << endl;
    return ;
  }
#endif

  memset(shmBuf, 0x00, SHMSZ);
  strncpy(shmBuf, shm, SHMSZ);
  std::string newBuf(shmBuf);
  newBuf += "|";
  newBuf += query;
  newBuf += "|";
  newBuf += value;

  //newBuf.length >= SHMSZ -> delete cache
  strcpy(shm, newBuf.c_str());

#if defined(WIN32)
	UnmapViewOfFile(shm);
#else
  shmdt(shm);
#endif

  // SEM unlock
#if defined(WIN32)
	ReleaseSemaphore(semid_, 1, NULL);
#else
  // semVal -> 0
  semBuf.sem_num = 0;
  semBuf.sem_op = -1;
  semBuf.sem_flg = 0;
  if (-1 == semop(semid_, &semBuf, 1)) {
    cerr << TAG_ERR << CLASS_SIG << "::addShm: semop down failed" << endl;
    return;
  }
#endif
}

#if defined(WIN32)
int
SrvConfAttrMgr::createShm()
{
	//lock
	WaitForSingleObject(semid_, INFINITE);
	shmMap = CreateFileMapping((HANDLE)0xFFFFFFFF, NULL, PAGE_READWRITE,
								0, 196608, "SRVCONFATTRMGRSHM");
	if (!shmMap) {
	    cerr << TAG_ERR << CLASS_SIG << "::" << "createShm: "
		 << "CreateFileMapping failed. " << endl;
		ReleaseSemaphore(semid_, 1, NULL);
		return -1;
	}
	//unlock
	ReleaseSemaphore(semid_, 1, NULL);

	return 0;
}

void
SrvConfAttrMgr::deleteShm()
{
	//lock
	WaitForSingleObject(semid_, INFINITE);

	CloseHandle(shmMap);

	//unlock
	ReleaseSemaphore(semid_, 1, NULL);

}
#endif

