/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: Timeout.cpp,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#include <iostream>
#include <stdexcept>

#if defined(WIN32)
#include <process.h>
#include "StringUtil.h"
#else
#include <pthread.h>
#include <unistd.h>
#endif

#include "Timeout.h"

using namespace OpenSOAP;
using namespace std;

#if defined(WIN32)
int Timeout::mtCount_ = 0;
#endif

//===========================================
// RXgN^
//-------------------------------------------
// : ^CAEg(b) 
//===========================================
Timeout::Timeout(int timeout_second)
  : timeoutSecond_(timeout_second)
    , targetThread_(0)
{
  initMutex();
}

//===========================================
// RXgN^
//-------------------------------------------
// : ^CAEg(b) 
// : ^C}[ĎΏۃXbhID
//===========================================
Timeout::Timeout(int timeout_second, 
#if defined(WIN32)
				HANDLE
#else
				pthread_t
#endif
					thr_id)
  : timeoutSecond_(timeout_second)
    , targetThread_(thr_id)
{
  initMutex();
}

//===========================================
// fXgN^
//===========================================
Timeout::~Timeout()
{
  destroyMutex();
}

//===========================================
// mutex
//===========================================
void Timeout::initMutex()
{
#if defined(WIN32)
	if (mtCount_ > 10000) {
		mtCount_ = 0;
	}
	std::string mtId = "TIMEOUTMTX" + StringUtil::toString(mtCount_++);
	isRunning_mutex_ = CreateMutex(NULL, FALSE, mtId.c_str());
#else
  int status = pthread_mutex_init(&isRunning_mutex_, NULL);
  if (0 != status) {
    //throw OpenSOAPException();
    throw std::exception();
  }
#endif
}

//===========================================
// mutexj
//===========================================
void Timeout::destroyMutex()
{
#if defined(WIN32)
	CloseHandle(isRunning_mutex_);
#else
  int status = pthread_mutex_init(&isRunning_mutex_, NULL);
  if (0 != status) {
    //throw OpenSOAPException();
    throw std::exception();
  }
#endif
}

//===========================================
// ^CAEgΏۃXbhw
//-------------------------------------------
// : ^C}[ĎΏۃXbhID
//===========================================
void Timeout::setTargetThread(
#if defined(WIN32)
							HANDLE
#else
							pthread_t
#endif
								thr_t)
{
  targetThread_ = thr_t;
}

//===========================================
// ^CAEgԎw(b)
//-------------------------------------------
// : ^CAEg(b) 
//===========================================
void Timeout::setTimeout(int timeout_second)
{
  timeoutSecond_ = timeout_second;
}


//===========================================
// ^C}[
//-------------------------------------------
// : TimeoutCX^X̃|C^
//===========================================
#if defined(WIN32)
void
#else
void*
#endif 
Timeout::runTimeout(void* arg)
{
#ifdef DEBUG
  //debug
  cerr << "Timeout::runTimeout() called" << endl;
#endif

  //CX^X擾
  // X^eBbN\bĥ,ɂCX^X
  //   ăCX^XoɃANZX
  Timeout* that = (Timeout*)arg;
  if (NULL == that) {
    //error
    cerr << "invalid argument..." << endl;
  }

  //w莞ԋx~
#if defined(WIN32)
	Sleep(that->timeoutSecond_*1000);
#else
  sleep(that->timeoutSecond_);
#endif

  //-------------------------------
  //ȍ~̓^CAEg
  //-------------------------------
  //  ^CAEgOɏ̏ꍇ͏Lsleep()
  //  finish()Ŗ{Xbh~

  // bNݒ
#if defined(WIN32)
	WaitForSingleObject(that->isRunning_mutex_, INFINITE);
#else
  int status = pthread_mutex_lock(&(that->isRunning_mutex_));
  if (0 != status) {
    //error
    return NULL;
  }
#endif

  //sԃ`FbN
  if (that->isRunning_) {
    //debug
    cerr << "### pthread cancel call!! ###" << endl;

    //s~(^CAEg)
#if defined(WIN32)
	if (-1 == TerminateThread(that->targetThread_, 0)) {
      cerr << "thread cancel error..." << endl;
	}
	else {
#ifdef DEBUG
		cerr << "Timeout:: thread canceled." << endl;
#endif
	    that->targetThreadIsCanceled_ = true;
	}
#else
    status = pthread_cancel(that->targetThread_);
    if (0 != status) {
      cerr << "pthread cancel error..." << endl;
    }
    else {
      that->targetThreadIsCanceled_ = true;
    }
#endif
  }

#ifdef DEBUG
  // for debug
  else {
    cerr << "## Timeout::runTimeout : not running. may be finished ##" << endl;
  }
#endif

  //bN
#if defined(WIN32)
	ReleaseMutex(that->isRunning_mutex_);
#else
  status = pthread_mutex_unlock(&(that->isRunning_mutex_));
  if (0 != status) {
    //error
    return NULL;
  }
#endif

#ifdef DEBUG
  //debug
  cerr << "Timeout::runTimeout() return" << endl;
#endif

#if defined(WIN32)
	_endthread();
#else
	pthread_exit(NULL);
  return NULL;
#endif
}

//===========================================
//^CAEgĎJn
//-------------------------------------------
// ԂlF Xe[^X
//          O:     
//          OȊO: G[
//===========================================
int Timeout::start()
{
#ifdef DEBUG
  //debug
  cerr << "Timeout::start() called" << endl;
#endif

  //ΏۃXbhݒ`FbN
  if (0 == targetThread_) {
    cerr << "target thread not set" << endl;
    return -1;
  }

  // bNݒ
#if defined(WIN32)
	WaitForSingleObject(isRunning_mutex_, INFINITE);
#else
  int status = pthread_mutex_lock(&isRunning_mutex_);
  if (0 != status) {
    //error
    return -1;
  }
#endif

  // tOݒ
  isRunning_ = true;
  targetThreadIsCanceled_ = false;
  
  //^C}[XbhJn
#if defined(WIN32)
	timerThr_id_ = (HANDLE)_beginthread(runTimeout, 0, this);
#else
  status = pthread_create(&timerThr_id_, 
			   NULL,
			   runTimeout, // static method
			   this);   // static method ɃCX^Xn
#endif

  //bN
#if defined(WIN32)
	ReleaseMutex(isRunning_mutex_);
#else
  status = pthread_mutex_unlock(&isRunning_mutex_);
  if (0 != status) {
    //error
    return -1;
  }
#endif

  /*
  //pthread_t thr_id;
  status = pthread_create(&timerThr_id_, NULL,
			   runTimeout, this);
  */

#ifdef DEBUG
  //debug
  cerr << "Timeout::start() return" << endl;
#endif

  return 0;
}

//===========================================
//^CAEgĎI
//-------------------------------------------
// ԂlF Xe[^X
//          O:     
//          OȊO: G[
//===========================================
int Timeout::finish()
{
#ifdef DEBUG
  //debug
  cerr << "Timeout::finish() called" << endl;
#endif

  // bNݒ
#if defined(WIN32)
	WaitForSingleObject(isRunning_mutex_, INFINITE);
#else
  int status = pthread_mutex_lock(&isRunning_mutex_);
  if (0 != status) {
    //error
    return -1;
  }
#endif

  // isRunning_ ̃`FbNsȂĂɂ邩???
  if (isRunning_) {
    // ^CAEgOɏIȂ̂,
    // ^C}[Xbh𒆎~
#if defined(WIN32)
	TerminateThread(timerThr_id_, 0);
#else
    status = pthread_cancel(timerThr_id_);
    if (0 != status) {
      cerr << "Timeout::runTimeout() thread cancel error" << endl;
    }
#endif
    
    // tO
    isRunning_ = false;
  }

  //bN
#if defined(WIN32)
	ReleaseMutex(isRunning_mutex_);
#else
  status = pthread_mutex_unlock(&isRunning_mutex_);
  if (0 != status) {
    //error
    return -1;
  }
#endif

#ifdef DEBUG
  //debug
  cerr << "Timeout::finish() return" << endl;
#endif
  
  return 0;
}
 
// ^[QbgXbh^CAEgŃLZꂽۂH
// 
// true:  LZĂB
// false: LZĂȂB
// 
bool
Timeout::targetThreadIsCanceled()
{
  return targetThreadIsCanceled_;
}
//--- End of Timeout.cpp ---
