/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: XmlQuery2.cpp,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#include <iostream>
#include <string>

#include <stdexcept>

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif /* HAVE_CONFIG_H */

/* libxml2 */
#if defined(HAVE_LIBXML2_XMLVERSION_H) 
#include <libxml2/tree.h>
#include <libxml2/parser.h>
#else
#include <libxml/tree.h>
#include <libxml/parser.h>
#endif /* !HAVE_LIBXML2_XMLVERSION_H */

//  #include "ServerCommon2.h"
#include "ServerCommon.h"
#include "SSMLAttrMgrProtocol.h"
#include "XmlQuery2.h"

using namespace OpenSOAP;
using namespace std;

// for LOG
static const std::string CLASS_SIG = "XmlQuery2";

//DOM
XmlQuery2::XmlQuery2(const std::string& query)
  : next_(NULL)
  , isTarget_(false)
  , isMulti_(false)
{
  std::string queryStr(query);

  //NODE_DELMIT(="/")ɂm[hKw̐؂o
  std::string::size_type begIdx = 0;
  std::string::size_type endIdx = 0;

  begIdx = queryStr.find_first_not_of(NODE_DELMIT);
  if (begIdx == std::string::npos) {
#ifdef DEBUG
    cerr << "END" << endl;
#endif /* DEBUG */
    cerr << TAG_ERR << CLASS_SIG << "::" << CLASS_SIG
	 << ": invalid query data. query=[" << query << "]" << endl;
    throw std::exception();
  }
  
  endIdx = queryStr.find_first_of(NODE_DELMIT, begIdx);
  if (endIdx == std::string::npos) {
    endIdx = queryStr.length();
  }

  //̃CX^X(Kw)ŏf[^
  std::string curQuery = queryStr.substr(begIdx, (endIdx-begIdx));

#ifdef DEBUG
  cerr << "curQuery=[" << curQuery << "]" << endl;
#endif /* DEBUG */

  //̊Kwֈnf[^
  queryStr = queryStr.substr(endIdx);

#ifdef DEBUG
  cerr << "next queryStr=[" << queryStr << "]" << endl;
#endif /* DEBUG */

  //-------------------------------------------------
  //vf(Kw), next_ɃCX^Xo^
  //-------------------------------------------------
  begIdx = queryStr.find_first_not_of(NODE_DELMIT);
  if (begIdx != std::string::npos) {
#ifdef DEBUG
    cerr << "next Query create." << endl;
#endif /* DEBUG */

    next_ = new XmlQuery2(queryStr);
  }

  //f[^
  //OƑ̐؂o
  begIdx = curQuery.find_first_not_of(ATTR_DELMIT);
  if (begIdx == std::string::npos) {
    cerr << TAG_ERR << CLASS_SIG << "::" << CLASS_SIG
	 << ": invalid current query data. query=[" << curQuery << "]" << endl;
    throw std::exception();
  }

  endIdx = curQuery.find_first_of(ATTR_DELMIT, begIdx);
  if (endIdx == std::string::npos) {
    endIdx = curQuery.length();
  }
  //O
  std::string nameToken = curQuery.substr(begIdx, (endIdx-begIdx));
  addName(nameToken);

  for (;;) {
    begIdx = curQuery.find_first_not_of(ATTR_DELMIT, endIdx);
    if (begIdx == std::string::npos) {
#ifdef DEBUG
      cerr << "END" << endl;
#endif /* DEBUG */
      break;
    }
    endIdx = curQuery.find_first_of(ATTR_DELMIT, begIdx);
    if (endIdx == std::string::npos) {
      endIdx = curQuery.length();
    }
    std::string attrToken = curQuery.substr(begIdx, (endIdx-begIdx));

#ifdef DEBUG
    cerr << "attrToken=[" << attrToken << "]" << endl;
#endif /* DEBUG */

    addAttributes(attrToken);
  }


}

void XmlQuery2::addName(const std::string& nameToken)
{
#ifdef DEBUG
  cerr << CLASS_SIG << "::addName() arg=[" << nameToken << "]" << endl;
#endif /* DEBUG */

  //VAL_DELMIT(="=")ɂl̐؂o
  std::string::size_type begIdx = 0;
  std::string::size_type endIdx = 0;

  begIdx = nameToken.find_first_not_of(VAL_DELMIT);
  if (begIdx == std::string::npos) {
    cerr << TAG_ERR << CLASS_SIG << "::" << CLASS_SIG
	 << ": invalid name token data. nameToken=[" << nameToken << "]" << endl;
    throw std::exception();
  }
  
  endIdx = nameToken.find_first_of(VAL_DELMIT, begIdx);
  if (endIdx == std::string::npos) {
    endIdx = nameToken.length();
  }
  
  //O
  std::string name = nameToken.substr(begIdx, (endIdx-begIdx));

  //̐؂o
  begIdx = nameToken.find_first_not_of(VAL_DELMIT, endIdx);
  if (begIdx == std::string::npos) {
    //w薳
#ifdef DEBUG
    cerr << CLASS_SIG << "::addName: no name condition." << endl;
#endif /* DEBUG */

    //added proc. for get child tag names #2001/09/10
    if (QUESTION_MARK == name) {
      //Ώ
      isTarget_ = true;
      node_.setName(name);
    } 
    else if (MULTI_QUESTION_MARK == name) {
      //Ώ()
      isTarget_ = true;
      isMulti_ = true;
      //Ȍ̊mFɂMULTI_QUESTION_MARKgȂ
      node_.setName(QUESTION_MARK);
    }
    else {
      node_.setName(name);
    }
    return;
  }
  //O̒ʏݒ
  node_.setName(name);

  endIdx = nameToken.find_first_of(VAL_DELMIT, begIdx);
  if (endIdx == std::string::npos) {
    endIdx = nameToken.length();
  }
  std::string cond = nameToken.substr(begIdx, (endIdx-begIdx));

#ifdef DEBUG
  cerr << CLASS_SIG << "::addName: name cond=[" << cond << "]" << endl;
#endif /* DEBUG */

  if (QUESTION_MARK == cond) {
    //Ώ
    isTarget_ = true;
    node_.setValue(cond);
  } 
  else if (MULTI_QUESTION_MARK == cond) {
    //Ώ()
    isTarget_ = true;
    isMulti_ = true;
    //Ȍ̊mFɂMULTI_QUESTION_MARKgȂ
    node_.setValue(QUESTION_MARK);
  }
  else {
    node_.setValue(cond);
  }
}

void XmlQuery2::addAttributes(const std::string& attrToken)
{
#ifdef DEBUG
  cerr << CLASS_SIG << "::addAttributes() arg=[" << attrToken << "]" << endl;
#endif /* DEBUG */

  //VAL_DELMIT(="=")ɂl̐؂o
  std::string::size_type begIdx = 0;
  std::string::size_type endIdx = 0;
  
  begIdx = attrToken.find_first_not_of(VAL_DELMIT);
  if (begIdx == std::string::npos) {
    cerr << TAG_ERR << CLASS_SIG << "::" << CLASS_SIG
	 << ": invalid attr token data. attrToken=[" << attrToken << "]" << endl;
    throw std::exception();
  }
  
  endIdx = attrToken.find_first_of(VAL_DELMIT, begIdx);
  if (endIdx == std::string::npos) {
    endIdx = attrToken.length();
  }
  
  DataSet attrDataSet;

  //
  std::string attrName = attrToken.substr(begIdx, (endIdx-begIdx));
  attrDataSet.setName(attrName);

  //̐؂o
  begIdx = attrToken.find_first_not_of(VAL_DELMIT, endIdx);
  if (begIdx == std::string::npos) {
    //w薳
    cerr << TAG_ERR << CLASS_SIG 
	 << "::addAttributes: no attr condition. attr=[" 
	 << attrName << "]" << endl;
    throw std::exception();
  }

  endIdx = attrToken.find_first_of(VAL_DELMIT, begIdx);
  if (endIdx == std::string::npos) {
    endIdx = attrToken.length();
  }
  std::string cond = attrToken.substr(begIdx, (endIdx-begIdx));

#ifdef DEBUG
  cerr << CLASS_SIG << "::addAttributes: attr cond=[" << cond << "]" << endl;
#endif /* DEBUG */

  if (QUESTION_MARK == cond) {
    //Ώ
    isTarget_ = true;
    attrDataSet.setValue(cond);
  } 
  else if (MULTI_QUESTION_MARK == cond) {
    //Ώ()
    isTarget_ = true;
    isMulti_ = true;
    //Ȍ̊mFɂMULTI_QUESTION_MARKgȂ
    attrDataSet.setValue(QUESTION_MARK);
  }
  else {
    attrDataSet.setValue(cond);
  }

  //ǉ
  attributes_.push_back(attrDataSet);
}

XmlQuery2::~XmlQuery2()
{
  if (next_) { delete next_; }
}
    
//l̎擾
//擾łtrueԂ
//RootQuery  RootNoden
//int XmlQuery2::getValue(xmlDocPtr& doc, xmlNodePtr node)
int XmlQuery2::getValue(xmlDocPtr& doc, xmlNodePtr node,
		       std::vector<std::string>& values)
{
  //bool result = false;
  int result = -101; // -101 = target not found

  //̈ꎞێobt@
  std::string answer;
  //擾σtO
  //bool hasAnswer = false;

  int i = 0;

  while (NULL != node) {
    
    //check node name
    if (QUESTION_MARK == node_.getName()) {
      //request tag name 
      answer = (const char*)node->name;
    }
    else {
      if (0 != strcmp((const char*)node->name, node_.getName().c_str())) {
	//ΏۊOm[h
	node = node->next;
	continue;
      }

      //check node element
      if (!node_.isNullValue()) {

	//vfwL̏ꍇ
	xmlChar* element = xmlNodeListGetString(doc, node->xmlChildrenNode, 1);
	//NULL is no error, return ""
	//      if (NULL == element) {
	//	return -102; // -102 = wvf
	//      }
	if (QUESTION_MARK == node_.getValue()) {
	  //ړI̓Ȃ΃obt@ɐݒ
	  if (NULL == element) {
	    answer = "";
	  }
	  else {
	    answer = (const char*)element;
	  }
	  //hasAnswer = true;
	}
	else if (0 != strcmp((const char*)element, node_.getValue().c_str())) {
	  //`FbN
	  //ΏۊOm[h
	  node = node->next;
	  continue;
	}
      }
     
      //check attributes
      std::vector<DataSet>::const_iterator pos;
      bool isValidAttr = true;
      for (pos = attributes_.begin(); pos != attributes_.end(); ++pos) {
	
	xmlChar* attr = xmlGetProp(node, 
				   (const xmlChar*)pos->getName().c_str());
	if (NULL == attr) {
	  return -103; // -103 = w葮
	}
	if (QUESTION_MARK == pos->getValue()) {
	  //ړI̓Ȃ΃obt@ɐݒ
	  /*
	    if (hasAnswer) {
	    cerr << TAG_WRN << CLASS_SIG 
	    << "::getValue: answer alread set." << endl;
	    }
	  */
	  answer = (const char*)attr;
	  //hasAnswer = true;
	}
	else if (0 != strcmp((const char*)attr, pos->getValue().c_str())) {
	  //`FbN
	  //ΏۊOm[h
	  isValidAttr = false;
	  break;
	}
      } // end for ();
      if (!isValidAttr) {
	node = node->next;
	continue;
      }
      
      //܂ł΂̃m[h͏Ώ
      //qCX^Xֈn
      if (next_) {
	
	if (isTarget_) {
	  cerr << TAG_WRN << CLASS_SIG << 
	    "::getValue: current instance is target, but next_ has..." << endl;
	}
	return next_->getValue(doc, node->xmlChildrenNode, values);
      }
    }

    //o^
    values.push_back(answer);
    result = 0; // 0 = SUCCESS

    //񓚂łȂΏI
    if (!isMulti_) {
      break;
    }

    //̃m[h
    node = node->next;
    
    i++;
  }

#ifdef DEBUG
  cerr << "return =(" << result << ")" << endl;
#endif

  return result;
}

void XmlQuery2::spy() const
{
  cerr << CLASS_SIG << "::spy() #####" << endl;
  cerr << CLASS_SIG << "::node_:" << endl;
  node_.spy();

  cerr << "  isTarget_:" << isTarget_ << endl;
  cerr << "  isMulti_:" << isMulti_ << endl;

  cerr << CLASS_SIG << "::attributes_:" << endl;
  std::vector<DataSet>::const_iterator pos;
  for (pos = attributes_.begin(); pos != attributes_.end(); ++pos) {
    cerr << " -------------" << endl;   
    pos->spy();
  }
  
  if (next_) { next_->spy(); }
}


    
int XmlQuery2::attach(xmlDocPtr& doc, 
		      //xmlNodePtr parentNode,
		      xmlNodePtr node,
		      xmlNodePtr parent,
		      const std::string& value, 
		      bool updateFlg,
		      bool useNs,
		      const NameSpaceDef* nsDef,
		      int insPos)
{
  if (!node && !parent) {
    cerr << TAG_ERR << CLASS_SIG 
	 << "::attach: node and parent are null." << endl;
    return -1;
  }

  //xmlNodePtr node = parentNode->xmlChildrenNode;

  //bool result = false;
  int result = -101; // -101 = target not found

  //̈ꎞێobt@
  std::string answer;
  //擾σtO
  //bool hasAnswer = false;

  //int i = 0;

  if (QUESTION_MARK == node_.getName()) {
    //request tag name 
    //answer = (const char*)node->name;
    //̐VK^O쐬D
    
    //VKm[h쐬
    xmlNodePtr newNode = xmlNewNode(NULL, (xmlChar*)value.c_str());
    
    if (useNs) {
      xmlNsPtr newNs = NULL;
      if (NULL == nsDef) {
	//default
	xmlNsPtr* nsPtrList = xmlGetNsList(doc, (parent ? parent : node));
	if (nsPtrList) {
	  newNs = *nsPtrList;
	}
	//newNs = xmlNewNs(newNode, NULL, NULL);
      }
      else {
	if (NULL != parent) {
	  newNs = xmlSearchNsByHref(doc,
				    //(node ? node : parent),
				    parent,
				    (xmlChar*)nsDef->href.c_str());
	}
	if (!newNs) {
	  newNs = xmlNewNs(newNode, 
			   (xmlChar*)nsDef->href.c_str(),
			   (xmlChar*)nsDef->prefix.c_str());
	}
      }
      xmlSetNs(newNode, newNs);
    }
    if (node) {
      if (-1 == insPos) {
	xmlAddPrevSibling(node, newNode);
      }
      else {
	for (xmlNodePtr next = node->next; NULL != next; next = node->next) {
	  node = next;
	}
	xmlAddNextSibling(node, newNode);
      }
    }
    else if (parent) {
      xmlAddChild(parent, newNode);
    }
    else {
      cerr << TAG_ERR << CLASS_SIG 
	   << "::attach: node and parent are NULL."
	   << endl;
      return -1;
    }
    return 0;
  }

  if (!node_.isNullValue() 
      && QUESTION_MARK == node_.getValue()
      && false == updateFlg) {
    
    //VKm[h쐬
    xmlNodePtr newNode = xmlNewNode(NULL, (xmlChar*)node_.getName().c_str());
    //lݒ
    xmlNodeSetContent(newNode, (xmlChar*)value.c_str());
    
    //namespace
    if (useNs) {
      xmlNsPtr newNs = NULL;
      if (NULL == nsDef) {
	//default
	xmlNsPtr* nsPtrList = xmlGetNsList(doc, (parent ? parent : node));
	if (nsPtrList) {
	  newNs = *nsPtrList;
	}
	//newNs = xmlNewNs(newNode, NULL, NULL);
      }
      else {
	if (NULL != parent) {
	  newNs = xmlSearchNsByHref(doc,
				    //(node ? node : parent),
				    parent,
				    (xmlChar*)nsDef->href.c_str());
	}
	if (!newNs) {
	  newNs = xmlNewNs(newNode, 
			   (xmlChar*)nsDef->href.c_str(),
			   (xmlChar*)nsDef->prefix.c_str());
	}
      }
      xmlSetNs(newNode, newNs);
    }
    if (node) {
      //̊KẅԎn߂̃m[h̑Oɒǉ
      if (-1 == insPos) {
	xmlAddPrevSibling(node, newNode);
      }
      else {
	for (xmlNodePtr next = node->next; NULL != next; next = node->next) {
	  node = next;
	}
	xmlAddNextSibling(node, newNode);
      }

    }
    else if (parent) {
      xmlAddChild(parent, newNode);
    }
    else {
      cerr << TAG_ERR << CLASS_SIG 
	   << "::attach: node and parent are NULL."
	   << endl;
      return -1;
    }

    return 0;
  }

  //for update or attributes modify
  while (NULL != node) {
    
    //check node name

    if (0 != strcmp((const char*)node->name, node_.getName().c_str())) {
      node = node->next;
      continue;
    }
    
    //qvfȂ΂̃m[hKwΏ
    //if (!next_) {
    if (!node_.isNullValue()) {
      //nodeǉ̏ꍇ
      if (QUESTION_MARK == node_.getValue() && updateFlg) {
	xmlNodeSetContent(node, (xmlChar*)value.c_str());
	return 0;
      }
      else {
	//nodevfw肠
	xmlChar* element = xmlNodeListGetString(doc, node->xmlChildrenNode, 1);
	if (0 != strcmp((const char*)element, node_.getValue().c_str())) {
	  //`FbN
	  //ΏۊOm[h
	  node = node->next;
	  continue;
	}
      }
    }
    //
    //check attributes
    std::vector<DataSet>::const_iterator pos;
    bool isValidAttr = true;
    const DataSet* targetDataSet = NULL;
    for (pos = attributes_.begin(); pos != attributes_.end(); ++pos) {
      
      if (QUESTION_MARK == pos->getValue()) {
	//ړI̓Ȃ΃obt@ɐݒ
	//answer = (const char*)attr;
	//hasAnswer = true;
	targetDataSet = pos;
      }
      else {
	xmlChar* attr = xmlGetProp(node, 
				   (const xmlChar*)pos->getName().c_str());
	if (NULL == attr) {
	  return -103; // -103 = w葮
	}
	
	if (0 != strcmp((const char*)attr, pos->getValue().c_str())) {
	  //`FbN
	  //ΏۊOm[h
	  isValidAttr = false;
	  break;
	}
      }
    } // end for ();
    if (!isValidAttr) {
      node = node->next;
      continue;
    }
    //ݒ肾ꍇŏD
    if (targetDataSet) {
      //namespace
      xmlNsPtr newNs = NULL;	
      if (useNs) {
	if (NULL == nsDef) {
	  //default
	  xmlNsPtr* nsPtrList = xmlGetNsList(doc, (parent ? parent : node));
	  if (nsPtrList) {
	    newNs = *nsPtrList;
	  }
	  //newNs = xmlNewNs(node, NULL, NULL);
	}
	else {
	  newNs = xmlSearchNsByHref(doc,
				    node,
				    (xmlChar*)nsDef->href.c_str());
	  if (!newNs) {
	    newNs = xmlNewNs(node, 
			     (xmlChar*)nsDef->href.c_str(),
			     (xmlChar*)nsDef->prefix.c_str());
	  }
	}
	//xmlSetNs(newNode, newNs);
      }
      xmlSetNsProp(node, newNs,
		   (xmlChar*)targetDataSet->getName().c_str(),
		   (xmlChar*)value.c_str());

      return 0;
    } 

    //܂ł΂̃m[h͏Ώ
    //qCX^Xֈn
    if (next_) {
      if (isTarget_) {
	cerr << TAG_WRN << CLASS_SIG << 
	  "::attach: current instance is target, but next_ has..." << endl;
      }

      return next_->attach(doc, node->children, node,
			   value, updateFlg, useNs, nsDef,
			   insPos);
    }
    result = 0; // 0 = SUCCESS

    //񓚂łȂΏI
    if (!isMulti_) {
      break;
    }
    //̃m[h
    node = node->next;
    
    //i++;
  }

#ifdef DEBUG
  cerr << "return =(" << result << ")" << endl;
#endif

  return result;
}
